Puppet::Parser::Functions::newfunction(:func3,
  :arity => 2,
  :doc => "Blah blah, this is a lot of documentation that the ruby parser must deal with
  because documentation is part of what is loaded at runtime. Some functions have
  very little documentation, and some have quite a lot. This simulates documentation
  that is slightly longer than the shortest ones.") do |vals|
    # produces nil
end
