---
layout: default
built_from_commit: <%= sha %>
title: '<%= title %>'
canonical: "/puppet/latest/types/<%= type %>.html"
---

# <%= title %>

> **NOTE:** This page was generated from the Puppet source code on <%= now %>



<%= body %>

