---
layout: default
built_from_commit: <%= sha %>
title: Report Reference
toc: columns
canonical: "/puppet/latest/report.html"
---

# Report Reference

> **NOTE:** This page was generated from the Puppet source code on <%= now %>

<%= body %>
