$tmp = [
func3(x,y), func3(x,y), func3(x,y), func3(x,y), func3(x,y),
func3(x,y), func3(x,y), func3(x,y), func3(x,y), func3(x,y),
func3(x,y), func3(x,y), func3(x,y), func3(x,y), func3(x,y),
func3(x,y), func3(x,y), func3(x,y), func3(x,y), func3(x,y),
]
