---
layout: default
built_from_commit: <%= sha %>
title: Metaparameter Reference
toc: columns
canonical: "/puppet/latest/metaparameter.html"
---

# Metaparameter Reference

> **NOTE:** This page was generated from the Puppet source code on <%= now %>

<%= body %>
