$tmp = [
func4(x,y), func4(x,y), func4(x,y), func4(x,y), func4(x,y),
func4(x,y), func4(x,y), func4(x,y), func4(x,y), func4(x,y),
func4(x,y), func4(x,y), func4(x,y), func4(x,y), func4(x,y),
func4(x,y), func4(x,y), func4(x,y), func4(x,y), func4(x,y),
]
