# Blah blah, this is a lot of documentation that the ruby parser must deal with
# because documentation is part of what is loaded at runtime. Some functions have
# very little documentation, and some have quite a lot. This simulates documentation
# that is slightly longer than the shortest ones.
#
Puppet::Functions.create_function(:func4) do
  def func4(x,y)
  end
end